
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

class ApplicationController < ActionController::API
  include ActionController::Serialization
  include DeviseTokenAuth::Concerns::SetUserByToken
  include Pundit
  include PublicActivity::StoreController

  # tracking user in papertrail
  before_action :set_paper_trail_whodunnit

  rescue_from Exception do |exception|
    logger.fatal "The request raised an exception:"
    logger.fatal exception
    logger.fatal exception.backtrace.first(10).join("\n")
    unless response_body
      render status: :internal_server_error
    end
  end

  # check if client application is allowed to consumes the API.
  before_action :allow_client_application, if: -> { Feature.active?(:allow_client_application) }

  # Prevent CSRF attacks by raising an exception.
  # For APIs, you may want to use :null_session instead.
  # protect_from_forgery with: :null_session
  before_action :configure_permitted_parameters, if: :devise_controller?
  rescue_from Pundit::NotAuthorizedError, with: :user_not_authorized

  # ensure *current_user* can be called in PublicActivity models
  # helper_method :current_user
  # hide_action :current_user

  protected

  def configure_permitted_parameters
    registration_params = [:name, :email, :description, :teacher_id, :birthday, :avatar, :cover, :password, :password_confirmation, :current_password, :terms_of_service]
    devise_parameter_sanitizer.permit :sign_up, keys: registration_params
    devise_parameter_sanitizer.permit :account_update, keys: registration_params
  end

  private

  def allow_client_application
    app = Application.find_or_initialize_by(application_id: request.headers["PortalMEC-AppID"].to_s)
    user_not_authorized if app.domain != request.domain
  end

  def user_not_authorized
    render status: :unauthorized
  end
end
