
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

class V1::PackagesController < ApplicationController
  before_action :set_objects

  def link
    if @objects.blank?
      render status: :not_found
    else
      response = PackageService.link(@objects)

      if response.nil?
        render status: :not_found
      else
        render json: { url: response }, status: :found
      end
    end
  end

  private

  def set_objects
    allowed = PackageService.allowed_classes
    @objects = objects_params[:object].map do |o|
      next unless allowed.include? o['type']
      obj = o['type'].constantize.where(id: o['id']).first
      break if obj.blank?
      obj
    end
  end

  def objects_params
    params.require(:package).permit(object: [:type, :id])
  end
end
