
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

module Reputationable
  extend ActiveSupport::Concern

  included do
    delegate :count, to: :followers, prefix: true
  end

  def submitted_objects
    learning_objects.count
  end

  def reviews_score_average
    scores = []
    learning_objects.find_each(batch_size: 1000) do |object|
      scores << object.review_ratings_average
    end

    return 0.0 if scores.empty?

    scores.inject(0.0) { |acc, elem| acc + elem }.to_f / scores.size.to_f
  end

  def learning_objects_in_best_collections
    return 0.0 if collections.nil?

    collections.where('score >= ?', collections_avg_score).count
  end

  def learning_objects_best_score
    learning_objects.maximum(:score)
  end

  def learning_objects_average_score
    learning_objects.average(:score)
  end

  def review_approval_average
    return 0.0 if reviews.nil? || reviews.size < 5

    array = reviews.map(&:rates_percentage)
    array.inject(0.0) { |acc, elem| acc + elem } / array.size
  end

  def learning_objects_recently_submitted
    return 0.0 if learning_objects.nil?

    learning_objects.where('created_at >= ?', (Time.current - 2.months)).count
  end

  def collections_score_average
    return 0.0 if collections.nil? || collections.where(privacy: 'public').count.zero?

    collections.where(privacy: 'public').average(:score)
  end

  private

  def collections_avg_score
    Rails.cache.fetch("Collection/avg_score", expires_in: 23.hours) do
      Collection.average(:score)
    end
  end
end
