
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

module Trackable
  extend ActiveSupport::Concern
  include PublicActivity::Common
  include ActivitiesFilterService

  included do
    after_create :new_create_activity
    before_destroy :new_destroy_activity
    after_update :new_update_activity
  end

  def new_update_activity
    return nil if previous_changes.blank?
    return new_activity(:update) if ignore_changes == %w(updated_at)
    filtered = previous_changes.reject { |x| ignore_changes.include?(x) }
    new_activity(:update) unless filtered.empty?
  end

  def new_create_activity
    new_activity(:create)
  end

  def new_destroy_activity
    params = { name: self.try(:name)}
    new_activity(:destroy,params)
  end

  def new_activity(action,params=nil)
    create_activity(
      action,
      owner: activity_owner,
      recipient: activity_recipient,
      privacy: activity_privacy,
      parameters: params
    )
  end

  def ignore_changes
    %w(updated_at)
  end

  def activity_owner
    proc { |controller, model| model.try(:user) || model.try(:owner) || controller.try(:current_user) }
  end

  def activity_recipient
    proc { |_controller, model| model.try(:recipient) || model }
  end

  def activity_privacy
    proc { |_controller, model| model.try(:privacy) || model.try(:recipient).try(:privacy) || 'public' }
  end

end
