
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

class LearningObjectObaaSerializer < ActiveModel::Serializer
  cache key: 'learning_object_obaa', expires_in: 4.hours, except: [:likes_count, :views_count, :downloads_count]
#   "educational": {
#     "description": "? não existe no PortalMecHomologa"
#   },

  def general
    {
      title: object.name,
      description: object.description,
      keywords: object.tags.map(&:name).join(','),
      created_at: object.created_at,
      updated_at: object.updated_at,
      thumbnail: object.default_thumbnail,
      score: object.score,
      review_ratings_average: object.review_ratings_average,
      likes_count: object.likes_count,
      downloads_count: object.downloads_count,
      views_count: object.views_count,
      shares_count: object.shares_count,
      acessibility: ''
    }
  end

  def identifier
    {
      catalog: 'PortalMec',
      entry: object.id,
      language: object.language.try(:code)
    }
  end

  def technical
    {
      location: object.default_attachment.try(:retrieve_url),
      format: object.default_attachment.try(:mime_type)
    }
  end

  def rights
    {
      copyrightAndOhterRestrictions: 'true',
      cost: 'false',
      description: object.license.try(:name)
    }
  end

  def metametadata
    {
      identifier: {
        catalog: 'PortalMec',
        name: object.publisher.try(:name),
        entry_type: object.publisher.class.name,
        entry: object.publisher.try(:id)
      },
      contribute: {
        role: 'Criador',
        entity: object.author,
        date: object.created_at
      },
      metadataSchema: 'OBAAv1.0',
      language: object.language.try(:name)
    }
  end

  # attributes :id , :name, :created_at, :updated_at, :description, :author, :score, :likes_count
  attributes :id, :general, :identifier, :technical, :rights, :metametadata

  # has_many :tags
  # has_many :attachments
end
