
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

require_dependency 'dspace'
require 'mimemagic'

class DspaceUploadWorker
  include Sidekiq::Worker
  sidekiq_options queue: :dspace

  def perform(learning_object_id, media_path, attachment_id = nil, description = nil)
    # find learning object
    learning_object = LearningObject.where(id: learning_object_id).first

    return false if learning_object.blank?

    # Require dspace gem
    Bundler.require(*Rails.groups)

    # create attachment
    file = File.new(media_path, 'r')
    mime_type = MimeMagic.by_magic(file).try(:type)
    bitstream = dspace.items.add_bitstream(file, id: learning_object.id_dspace, name: File.basename(media_path), description: description)
    bitstream.mime_type = mime_type
    attachment = create_attachment(learning_object, bitstream, attachment_id)

    ThumbnailGenerateWorker.perform_async attachment.id
  ensure
    ActiveRecord::Base.clear_active_connections!
    dspace.logout
    dspace.close_connection
  end

  private

  def create_attachment(learning_object, bitstream, attachment_id = nil)
    unless attachment_id.nil?
      attachment = learning_object.attachments.where(id: attachment_id).first
      unless attachment.blank?
        if attachment.bundle_name == "TEMP" && File.exist?(attachment.retrieve_link)
          dir = File.dirname(attachment.retrieve_link)
          # Delete only the file if parent directory is uploads,
          # otherwise deletes the directory and files
          if dir =~ /uploads$/
            FileUtils.rm(attachment.retrieve_link, force: true)
          else
            FileUtils.rm_r(dir, force: true)
          end
        end
        attachment.update(map_bitstream2attachment(bitstream))
        return attachment
      end
    end
    learning_object.attachments.create map_bitstream2attachment(bitstream)
  end

  def map_bitstream2attachment(bitstream, options = {})
    ::Dspace::AttachmentMapper.call bitstream, options
  end

  def dspace
    @dspace ||= DspaceService.create_client
    counter = 0
    loop do
      status = @dspace.status

      if status["authenticated"]
        break
      else
        counter += 1
        Sidekiq.logger.fatal("Lost DSpace authentication. Attempt ##{counter}")
        @dspace.close_connection
        sleep(30.seconds)
        @dspace = DspaceService.create_client
        break if counter == 5
      end
    end
    @dspace
  end
end
