
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

require_dependency 'dspace'
require 'rails'
class PackageWorker
  include Sidekiq::Worker
  require 'zip'

  sidekiq_options queue: :package_cache

  def perform(objects_ids = nil, filename = nil)
    return false if objects_ids.blank? || filename.blank?
    Bundler.require(*Rails.groups)

    return true if File.exist?(PackageService.file_root(filename))
    @parc=""
    files = open_files(objects_ids)
    create_package(filename, files)
  ensure
    close_files(files) unless files.nil?
    ActiveRecord::Base.clear_active_connections!
    dspace_client.logout
    dspace_client.close_connection
  end

  private

  def open_files(objects_ids)
    objects = objects_ids.map { |o| o['class'].constantize.where(id: o['id']).first }

    files = []
    objects.each do |object|
      if object.is_a? Collection
        object.learning_objects.each { |o| files.concat object_files(o) }
      else
        files.concat object_files(object)
      end
    end
    
    if !@parc.blank?
      @parc = "Se você está vendo esse arquivo significa que você tentou baixar conteúdos que não estão hospedados em nossos servidores, entretanto como essa ação não é possível você pode acessar esses conteúdos com os links abaixo:\n\n#{@parc}"
      tx = File.new(Rails.root.join("tmp",'links.txt'), 'w+')
      tx.write(@parc)
      tx.close
      tx = File.open(Rails.root.join("tmp",'links.txt'), 'r')
      files << tx
    end

    files
  end

  def object_files(object)
    files = []
    attachments = object.attachments.where(bundle_name: 'ORIGINAL')
    if attachments.empty? && !object.link.nil?
      @parc += "#{object.name}: #{object.link}\n"
    else
      attachments.each do |a|
        link = a.retrieve_cache_link
        link = a.id_dspace.to_s if link =~ %r{^https?:\/\/.*}i
        file = open_file(link)
        files << file unless file.nil?
      end
    end

    files
  end

  def create_package(filename, files)
    FileUtils.mkdir_p(PackageService.file_root)
    Zip::File.open(PackageService.file_root(filename), Zip::File::CREATE) do |zipfile|
      files.each { |file| zipfile.add(File.basename(file.path), file.path) }
    end
    FileUtils.chown_R 'portalmec', 'portalmec', PackageService.file_root.to_s, verbose: true
  rescue => e
    file = PackageService.file_root(filename)
    FileUtils.rm(file) if File.exist?(file)
    raise e
  end

  def open_file(file_path)
    path = Rails.root.join("public" + file_path)
    return File.open(path) if File.exist? path
    open_dspace_file(file_path)
  end

  def open_dspace_file(id)
    file = dspace_client.bitstreams.retrieve(id: id)
    unless file.nil?
      file.open
      FileUtils.chmod 0664, file.path
    end
    file
  end

  def close_files(files)
    files.each do |file|
      path = file.path.to_s
      file.close
      FileUtils.rm(path) if path =~ %r{^\/tmp\/.*} && File.exist?(path)
    end
  end

  def dspace_client
    @dspace_client ||= DspaceService.create_client
  end
end
