
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

class ThumbnailGenerateWorker
  include Sidekiq::Worker
  sidekiq_options queue: :thumbnail

  def perform(attachment_id, media_path = nil, force = false)
    @attachment = ::LearningObject::Attachment.where(id: attachment_id).first
    return false if @attachment.blank? || (!@attachment.thumbnail.blank? && !force) || @attachment.bundle_name == "LINK"

    # object from dspace
    if media_path.nil?
      if ["ORE", "LINK", "TEMP"].include? @attachment.bundle_name
        media = create_media(@attachment.retrieve_link)
      end
      media = bitstream_download(@attachment.id_dspace)
    # new object
    else
      media = create_media(media_path)
    end
    return false if media.blank?
    create_thumbnail(media)
  ensure
    ActiveRecord::Base.clear_active_connections!
    dspace_client.logout
    dspace_client.close_connection
  end

  private

  def create_media(path)
    return nil if path.blank? || path =~ URI::regexp
    # return ::Screencap::Fetcher.new(path) if path =~ URI::regexp
    File.open path
  end

  def create_thumbnail(media)
    thumbnail = thumbnail_service.create_thumbnail(media)
    return false if thumbnail.blank?

    @attachment.update(thumbnail: thumbnail)
    File.unlink(thumbnail) if File.exist?(thumbnail.path)
    File.unlink(media) if File.exist?(media.path)
  end

  def bitstream_download(id)
    bitstream = dspace_client.bitstreams.retrieve(id: id)
    return nil if !((bitstream.is_a? File) || (bitstream.is_a? Tempfile))
    bitstream.open
  end

  def thumbnail_service
    @@thumbnailService ||= ThumbnailService.new(generators)
  end

  def generators
    [
      # Deactivated thumbnails for videos and URLs
      # Thumbnail::Strategies::VideoThumbnailGenerator.new,
      #Thumbnail::Strategies::UrlThumbnailGenerator.new,
      Thumbnail::Strategies::ImageThumbnailGenerator.new,
      Thumbnail::Strategies::PdfThumbnailGenerator.new
    ]
  end

  def dspace_client
    @dspace_client ||= DspaceService.create_client
  end
end
