
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

require 'fileutils'

  namespace :attachment_maintaining_service do

    desc 'Removing attachments and packages using random politic'
    task :remove_attachments, [:limit] => :environment do |t, args|
      args.with_defaults(limit: 10.gigabyte)
      attachments_dir = attachment_dir + "/*"
      packages_dir = package_dir + "/*"

      while dir_size(attachment_dir) > args.limit.to_i do
        # TODO now the politic is random, verify if politics like LRU or NRU improve the efficiency
        FileUtils.rm_rf Dir[attachments_dir].sample
      end

      while dir_size(package_dir) > args.limit.to_i do
        # TODO now the politic is random, verify if politics like LRU or NRU improve the efficiency
        FileUtils.rm_rf Dir[packages_dir].sample
      end
    end

    task check_links: :environment do
      OpenSSL::SSL::VERIFY_PEER = OpenSSL::SSL::VERIFY_NONE
      start = 1
      finish = 999
      batch_size = 1000
      last_id = LearningObject::Attachment.count > 0 ? LearningObject::Attachment.last.id : 0

      loop do
        begin
          p "start: #{start}, finish: #{finish}"
          LearningObject::Attachment.find_each(start: start, finish: finish) do |att|
            att_url = att.retrieve_url
            if att_url.blank?
              p "-----------------------------------------------------"
              p "Retrieve URL nil"
              p "LearningObject id: #{att.learning_object.id}" if !att.learning_object.nil?
              p "Attachment id: #{att.id}, id_dspace: #{att.id_dspace}"
            else
              begin
                URI.parse(att_url)
              rescue URI::InvalidURIError              
                p "-----------------------------------------------------"
                p "Invalid URI: #{att_url}"
                p "LearningObject id: #{att.learning_object.id}" if !att.learning_object.nil?
                p "Attachment id: #{att.id}, id_dspace: #{att.id_dspace}"
                next
              end
              uri = URI(att_url)
              response = Net::HTTP.get_response(uri)
              if !response.kind_of? Net::HTTPSuccess
                p "-----------------------------------------------------"
                p "Response != 200: #{response.inspect}"
                p "Link: #{att_url}"
                p "LearningObject id: #{att.learning_object.id}" if !att.learning_object.nil?
                p "Attachment id: #{att.id}, id_dspace: #{att.id_dspace}"
              end
            end
          end
        rescue Exception => e
          #logger.warn 'Database error, going to sleep'
          p 'Database error, going to sleep'
          #logger.error e
          p e
          p e.class
          p e.message
          ActiveRecord::Base.clear_active_connections!
          # Sleeps for a while to wait database's recovery
          sleep(60.seconds)
          # Goes to next iteration to retry
          next
        else
          start += batch_size - 1
          finish += batch_size
          break if start >= last_id
        end
      end
    end

    private

    def dir_size(dir)
      `du -sb #{dir} | cut -f 1`.to_i
    end

    def attachment_dir
      Rails.root.join('public','attachments').to_s
    end

    def package_dir
      Rails.root.join('public','packages').to_s
    end

  end