
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

require 'acceptance_helpers'
require 'shared/contexts'

resource 'Follows' do

  explanation "An User can follow a Collection or another User to see their Activities."

  before { 12.times { create(:user) } }

  let(:users) { User.all }

  post '/v1/:type/:id/follow' do
    include_context "authenticate_user"

    parameter :type, 'Type of object (users or collections)'
    parameter :id, 'The id of the object to follow'

    let(:type) { 'users' }
    let(:id) { users.first.id }

    example_request 'Following' do
      expect(status).to eq(201)
    end
  end

  delete '/v1/:type/:id/follow' do
    include_context "authenticate_user"

    parameter :type, 'Type of object (users or collections)'
    parameter :id, 'The id of the object to follow'

    let(:type) { 'users' }
    let(:id) { users.first.id }

    before do
      create(:follow, user: @user, followable: users.first)
    end

    example_request 'Unfollowing' do
      expect(status).to eq(200)
    end
  end

end
