
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

require 'rails_helper'

RSpec.describe V1::CollectionsController, type: :controller do
  fixtures :all

  it { should use_before_action(:authenticate_user!) }
  it { should use_before_action(:set_collection) }
  it { should route(:get, '/v1/collections').to(action: :index) }

  # authenticate the application before all tests
  before do
    auth_application applications(:test_app)
  end

  describe 'sociable actions' do
    it { should use_before_action(:authorize_sociable!) }
    it { should use_before_action(:view_object!) }
    it { should route(:post, '/v1/collections/1/like').to(action: :like, id: 1) }
    it { should route(:delete, '/v1/collections/1/like').to(action: :unlike, id: 1) }

    describe 'POST #like' do
      context 'when user is authenticated' do
        it do
          auth_user users(:admin)
          post :like, id: collections(:ufpr).id
          expect(response).to have_http_status(:created)
        end
      end

      context 'when user is not authenticated' do
        it do
          post :like, id: collections(:ufpr).id
          expect(response).to have_http_status(:unauthorized)
        end
      end
    end
  end

  describe 'GET #index' do
      before do
        get :index
      end

      it { expect(response.content_type).to eq('application/json') }
      it { expect(response).to have_http_status(:ok) }
  end

end
