
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

FactoryBot.define do
  factory :learning_object, aliases: [:downloadable, :reviewable, :stageable, :subjectable, :likeable, :collectionable, :bookmarkable] do
    sequence(:name) { |i| "LearningObject #{i}" }
    sequence(:id_dspace) { |i| i }
    publisher
    score { rand(0.0..1.0) }
    description { Faker::Lorem.paragraph }
    object_type { ObjectType.all.sample }
    language { [Language.all.sample] }
    author { Faker::Name.name }
    state { LearningObject.states[:published] }
    
    transient do
      tags_count { 5 }
      subjects_count { 2 }
      educational_stages_count { 1 }
      reviews_count { 2 } 
    end

    after(:create) do |learning_object, evaluator|
      create_list(:tagging, evaluator.tags_count, taggable: learning_object)
      create_list(:review, evaluator.reviews_count, reviewable: learning_object)
      [Subject, EducationalStage].each do |model|
        table = model.to_s.tableize
        learning_object.try(table.to_sym) << model.all.sample(evaluator.try("#{table}_count".to_sym))
      end
    end
  end
end
